                            %% SCRIPT OBJECTIVES

% Estimate the flux of the gas across the air-water interface.

                        %% FLUX NUMERICAL OPTIONS

%OBJECTIVES: declares the numerical options to the flux calculus functions  

% Flux numerical scheme: 'Mol' or 'ppm'
NSflux = 'ppm';

% Surface Layer numerical scheme: 'Double' or 'Single'
NSlayer = 'Double';

% Flux units: (1) mol*m^-2*s^-1; (2) mmol*m^-2*h^-1; (3) mmol*m^-2*d^-1
NSfluxunits = 3;

% Henry constant correction for temperature and salinity
% 'Sar13': Sarmiento (2013) adapted from Weiss (1974), Weiss and Price (1980)
% 'Joh10': Johnson (2010)
NSkH = 'Joh10';

% Air transfer velocity (ka)
% 'Joh10': Johnson (2010)
% 'Dea91': Duce et al (1991)
% 'Lis73': Liss (1973)
% 'M&Y83': Mackay & Yeun (1983), for wind speeds between 5 and 22 m/s
% 'Sea02': Shahin et al 2002
% 'Jef10': COARE's Jeffrey et al (2010)
NSka = 'Jef10';

% Friction velocity from Eddy Covariance measurements
% 'E&F98': Edson & Fairall(1998), implemented in COARE 2.5
% 'Frl03': Fairall et al. (2003), implemented in COARE 3.0
% 'Lea04': Lange et al (2004)
NSEC2ustar = 'Frl03';

% Drag coefficient
% 'Dea91': Duce et al (1991)
% 'Don97': Donelan et al (1997)
% 'M&Y83': Mackay & Yeun (1983)
% 'Smi80': Smith (1980)
% 'T&Y01': Taylor and Yelland (2001)
NSdragcoef = 'Dea91';

% Roughness length
% 'Chk55': Charnook (1955)
% 'Ck_cp': Variable z_Charnock dependent on wave age (c_p) and considering the constants A_Ck and B_Ck provided below
% 'TY01a': Taylor and Yelland (2001) \ equation 5
% 'TY01b': Taylor and Yelland (2001) \ equation 7 adapted for u*
% 'Hwg05': Hwang (2005) dependent on wave age (c_p) and wave length (Lp)- adapted for all water depths 
% 'PanZ0': Pan et al (2008) for z0 from wave age
% 'PanZC': Pan et al (2008) for z_Ch from wave age
NSz0 = 'TY01a';

% method to estimate friction velocity (u*)
% 'WLLP': from Wind Log-Linear Profile
% 'iWLP': from Wind Log-Linear Profile iterated to convergence following COARE
% 'VHFx': from Vertical Heat Flux term in Monin-Obhukov Similarity theory
NSu_='iWLP';

% number of iterations in case of iterative Wind Log-Linear Profile method
niWLLP = 4;

% method to estimate the smooth flow for the iWLP calculations of u_ and z0
% 'Smi88': Smith (1988), also used by Lange et al (2004), and Fairall et al (2003) in Coare 3.0
% 'Aea11': Andreas et al (2011)
% 'none.': no smooth flow implementation
NSsmoothflow = 'Smi88';

% method to estimate the Monin-Obhukov Stability theory:
% 'COARE' : the COARE equations for zeta without iterations for convergence
% 'L97Bu' : Lee (1997) adapted to Businger et al (1971)
% 'L97Dy' : Lee (1997) adapted to Dyer (1974)
% 'L97DG' : Lee (1997) adapted to Delage and Girard (1992)
% 'Lea04' : Lange et al(2004)
% 'Stu88' : Stull (1988)
NSmos = 'Stu88';

% method to estimate the bulk Richardson number:
% 'G&F97' : Grachev and Fairall (1997)
% 'Lee97' : Lee (1997)
% 'Stu88' : Stull (1988)
NSrib ='G&F97';
    
% Water transfer velocity (kwind)
% 'Bea04': Borges et al (2004)
% 'Cea96': Carini et al (1996)
% 'Ho+06': Ho et al (2006) upper boundary
% 'Ho-06': Ho et al (2006) lower boundary
% 'Jac99': Jacob et al (1999)
% 'Jea87': Jhne et al (1987), dependent on u_
% 'L&M86': Liss & Merlivat (1986)
% 'M&Y83': Mackay & Yeun (1983), dependent on u_
% 'MY83W': Mackay & Yeun (1983) adapted from Wolff and Heijde (1982), dependent on u_
% 'McG01': McGillis et al (2001)
% 'Nea00': Nightingale et al (2000)
% 'R&C01': Raymond & Cole (2001)
% 'Sea07': Sweeney et al (2007)
% 'Wan92': Wanninkhof (1992)
% 'WMG99': Wanninkhof & McGillis (1999)
% 'Wea07': Weiss et al (2007)
% 'Wea09': Wanninkhof et al (2009)
% 'Zea03': Zhao et al (2003), dependent on u_
% 'ZoJ03': Zhao et al (2003) on data from Jhne et al (1985), dependent on u_
% 'none.': no kwind term
NSkwind = 'Jea87';

% Water-side transfer velocity bubble term (kbubble)
% 'W05va': Woolf (2005)        with kinematic viscosity of air, dependent on u_and whitecap. Use together with kwind = Jea87
% 'W05vw': Woolf (2005)        with kinematic viscosity of water, dependent on u_ and whitecap. Use together with kwind = Jea87
% 'Zhg06': Zhang et al (2006)  dependent on u10 and whitecap. Use together with kwind = WMG99
% 'ZRb03': Zhao et al (2003)   dependent on u_ and whitecap. Use together with kwind = none.
% 'Z&X10': Zhao & Xie (2010)   dependent on u_ and whitecap. Use together with kwind = none.
% 'none.': no kbubble term
NSkbubble = 'W05va';

% Water-side transfer velocity current term (kcurrent)
% 'OCD58': O'Connor and Dobbins (1958) dependent on current and depth
NSkcurrent = 'OCD58';

% Use chemical enhancement (from Wanninkhof, 1992)for sparingly soluble gases subject to chemical reaction with water ('yes' or 'no.')
chemen ='no.';

% Dependence of water viscosity from temperature and salinity (alfa)
% 'Bea04': Borges et al (2004)
% 'Cea96': Carini et al (1996)
% 'Joh10': Johnson (2010)
% 'R&C01': Raymond Cole (2001)
NSalfa = 'Joh10';

% Water difusivity scheme (Dw)
% 'H&L74': Hayduk & Laudie (1974)
% 'H&M82': Hayduk & Minhas (1982)
% 'W&C55': Wilkie & Chang  (1955)
NSdw = 'W&C55';

%Stores numerical options in a structure array (not to edit)
NumericScheme.alfa       = NSalfa;
NumericScheme.chem_enh   = chemen;
NumericScheme.Dw         = NSdw;
NumericScheme.DragCoef   = NSdragcoef;
NumericScheme.Flux       = NSflux;
NumericScheme.FluxUnits  = NSfluxunits;
NumericScheme.kH         = NSkH;
NumericScheme.ka         = NSka;
NumericScheme.kbubble    = NSkbubble;
NumericScheme.kcurrent   = NSkcurrent;
NumericScheme.kwind      = NSkwind;
NumericScheme.Layer      = NSlayer;
NumericScheme.MOS        = NSmos;
NumericScheme.Rib        = NSrib;
NumericScheme.u_         = NSu_;
NumericScheme.z0         = NSz0;
NumericScheme.smoothflow = NSsmoothflow;


                              %% CONSTANTS

%OBJECTIVES: provides the constants for the gas flux model.

%--------------------------------------------------------------------------
                             %Air constants

% Specific Heat of air                                                     J/(g*K)
cp = 1.005;                                                          

% gravitational acceleration                                               m/s^2
g = 9.80665;                                                           

% von Krmn's constant for friction velocity estimates
Karman = 0.4;

% depth of convective boundary layer (CBL) for COARE estimation of
% saturation Ri_b under unstable conditions                                m
zCBL = 600;

% bulk transfer coefficient for sensible heat for COARE estimation of
% saturation Ri_b under unstable conditions                                scalar
Ch = 0.003;

% beta for COARE estimation of saturation Ri_b under unstable conditions   scalar
bCOARE = 1.25;

% roughness lenght bounds [z0min z0max].                                   m
% z0min to be used upon the lack of the wave data required for the estimation of z0
% z0max to stabilize the algorithm preventing the z0 estimation to blow and blow everything thereafter 
z0bounds = [0.00000001 0.01];

% Critical roughness Reynolds number ([Re_min Re_max]) for smooth flow (<Re_min) or rough flow (>Re_max) of the air over the water surface
% Re_min = 0.1  , proposed by Fairall et al (2000)
% Re_min = 0.11 , proposed by Donelan (1990)
% Re_min = 0.135, proposed by Andreas et al (2011)
% Re_max = 1    , proposed by Fairall et al (2000)
% Re_max = 2    , proposed by Wu (1981)
% Re_max = 2.3  , proposed by Toba et al (1990), Johnson et al (1998), Drennan et al (2003), Lange et al (2004), and Andreas et al (2011)
% Re_max = 2.33 , proposed by Wu (1980)
% Re_max = 3    , proposed by Kitaigorodskii (1970) 
Re_c = [0.11 2.3];

% Relative molecular mass of air
Ma = 28.97;

% Rho air parameters (in Johnson 2010)
Sd = [1.293393662 -5.538444326*10^-3 3.860201577*10^-5 -5.2536065*10^-7];

% Eta air parameters (in Johnson 2010)
Sv = [1.715747771*10^-5 4.722402075*10^-8 -3.663027156*10^-10 ...
      1.873236686*10^-12 -8.050218737*10^-14];

% Liquid molar volume of air                                               cm^3/mol
Va = 20.1;                                                          

%--------------------------------------------------------------------------
                            %Water constants

% A_Ck and B_Ck constants for roughness length (z0) estimated from wave age following Charnock (1955)
% 0.02     0.5    Toba et al    (1990)
% 0.8      -1     Maat et al    (1991)
% 0.48     -1     Smith et al   (1992)
% 2.87     -1.69  Monbaliu      (1994)
% 1.89     -1.59  Johnson et al (1998)
% 49.9     -2.52  Oost et al    (2001)
% 1.7      -1.7   Drennan et al (2003)
A_Ck = 0.48;    B_Ck=-1;

% Aw and Bw constants for roughness length (z0) estimated from wave height (Hs) and wave length (Lp) following to Taylor and Yelland (2001)
Aw = 1200;                                                                 % 1200 = original value by Taylor and Yelland (2001)
Bw = 4.5;                                                                  % 4.5  = original value by Taylor and Yelland (2001). 1.2 is value for our tune of Taylor and Yelland (2001) formulation with wave age.
Cw = 3.5;

% Relative molecular mass of H20
Ms=18.01;

% Mass fraction (MF) of solutes in standard sea water as a proportion of total salinity.
MassFraction= ...
{'Compound' 'MF'  'v1'     'v2'      'v3'     'v4'       'v5'      'v6'   ;
  'NaCl'    0.798  16.22    1.3229    1.4849   0.0074691  30.78    2.0583 ;
  'KCl'     0.022   6.4883  1.3175   -0.7785   0.09272    -1.3     2.0811 ;
  'CaCl2'   0.033  32.028   0.787792 -1.1495   0.0026995  780860   5.8442 ;
  'MgCl2'   0.047  24.032   2.2694    3.7108   0.021853   -1.1236  0.14474;
  'MgSO4'   0.100  72.269   2.2238    6.6037   0.0079004  3340.1   6.1304};

% Association factor of H20
phiw = 2.6;

% Rhow0 (pure water) parameters (in Johnson 2010)
r = [999.842594 0.06793952 -0.00909529 0.0001001685 -0.000001120083 ...
                                                        0.000000006536332];

% Rhow (marine water) parameters (in Johnson 2010)
a = [0.824493 -0.0040899 0.000076438 -0.00000082467 0.0000000053875];
b = [-0.00572466 0.00010277 -0.0000016546];
c = 0.00048314;

%--------------------------------------------------------------------------
                            %Gas constants
                               
%'a' and 'b' constants for temperature and salinity dependence of Bunsen solubility coeficient taken from Sarimiento and Gruber (2013)
as = [-160.7373   215.4152  89.892    -1.477592]; %for CO2
bs = [ 0.029941  -0.027455  0.0053407          ]; %for CO2
%as = [-68.8862    101.4956  28.7314   0        ]; %for CH4
%bs = [-0.076146   0.04397  -0.0068672          ]; %for CH4
%as = [-165.8806   222.8743  92.0792   -1.48425 ]; %for N2O
%bs = [-0.056235   0.031619 -0.0048472          ]; %for N2O
                            
% Henry's law constant in pure water at 25Celsius. K=c/p form.            mol/(L.atm)
kHcp = 0.034;      %(for CO2)
%kHcp = 0.0014;    %(for CH4)
%kHcp = 0.025;     %(for N2O)

% Relative molecular mass
Mb = 44.01;           %(for CO2)
%Mb = 16.043;           %(for CH4)
%Mb = 44.013;           %(for N2O)

% Universal ideal gas law constant                                         Pa.m^3/(mol.K)
R = 8.314472;

% Temperature dependence of the solution: -delta_soln*H/R (in Johnson 2010)
tds = 2400;         %(for CO2)
%tds = 1700;         %(for CH4)
%tds = 2600;         %(for N2O)

% Liquid molar volume at its boiling point using the additive Schroeder method:
% increments of C=7,O=7, N=7, H=7, double bound=7 and triple bound=14 (see Johnson 2010)
% for CO2: molecular structure of CO2 is O=C=O                             cm^3/mol
Vb = 1*7+2*7+2*7;
% for CH4: molecular structure of CH4 is 1C to 4H with 4 single bounds
%Vb = 1*7+4*7;
% for N2O: molecular structure of N2O is N=N=O
%Vb = 1*7+2*7+2*7;

%Real gas molar volume                                                     L/mol
Vm = 22.3;  %for CO2 and CH4
%Vm = 22.2432;  %for N2O
                    
                              
                            %% Variables

%OBJECTIVES: provides the environmental variables to run the analysis
                            
dir='C:\Users\Vasco Vieira\Dropbox\InterfaceGasFlux\manuscripto 3 (Atmosphere)\Supplementary Files\'; %directory with the data file
file='L2 data';                                                            %name of the data file
load([dir file]);
   
% Gas concentration in the air in                                          mol/m^3
Xa = NaN(size(pa));

% Gas concentration in the air in                                          ppm
pXa = Ca_ppm;

% Air temperature at height 0                                              C
Ta0 = ta;%T0m;

% Air temperature at height h                                              C
Tah = T12m;

% Air pressure at height 0                                                 atm
P0  = pa/1000;%P./101325;

% Air pressure at height h                                                 atm
Ph  = pa/1000;%P./101325;

% relative humidity at height h                                            scalar
hrh = [];

% specific humidity at height h                                            g/Kg
qh  = Q;

% liquid-water saturation mixing ratio at height 0                         scalar
rL0 = [];

% liquid-water saturation mixing ratio at height h                         scalar
rLh = [];

% height h for T, P and q measures                                         m
hTP = zu;

% Wind velocity h meters above the surface                                 m/s
U = WS;%Uz;

% heigth of wind measurement                                               m
zu = zu;

% Roughness length                                                         m
z0 = [];

% friction velocity (wind generating shear stress at the sea surface)      m/s
u_ = [];

% friction velocity u'w' component from E-C data                           m/s
uw = [];%uw;

% friction velocity v'w' component from E-C data                           m/s
vw = [];%vw;

% Vertical Heat Flux                                                       J.m^-2.s^-1
vhf = [];

% Gas concentration in the water                                           mol/m^3
Xw = nan(size(pa));

% Gas concentration in the water in equivalent air partial pressure when in
%equilibrium. Optionals: fixed temperature and/or salinity.                ppm
pXw = Cw_ppm;

% Significant wave heigth (wind seas)                                      m
Hs = Hs;

% Wave length of waves at the peak wave spectrum (wind seas)               m
Lp = Lp;

% Wave frequency (wind seas)                                               rad/s
fp = fp*2*pi;

% psi_m (from Atmospheric Stability)                                       scalar
psi_m = [];

% Salinity                                                                 ppt
S = S;

% Water skin temperature                                                   C
Tw = Tw;%TwDWR;

% Water current velocity                                                   m/s
w = [];

% Water depth                                                              m
zw = [];

%clear useless arrays
clear date dir file

                               %% CALCULUS

%The sections below run the calculus. Do not change them as it may stop the %software from working properly.

%OBJECTIVES: check for errors----------------------------------------------
i = sum(find(zu~=10));
if sum(strcmp(NSka,{'Lis73' 'Dea91' 'Sea02'}))>=1 && i>0
    'error: ka numerical scheme requires 10m wind measurement heigth'
end
    
%OBJECTIVES: estimates solubility and converts gas concentration units-----
Tka   = 273.15+Ta0;
Tkw   = 273.15+Tw;
pH2OP = exp(24.4543-67.4509*(100./Tkw)-4.8489*log(Tkw./100)-0.000544*S);
pXa   = pXa.*(1-pH2OP);

%Conversion of gas concentration in the air from partial pressure (ppm) to mol/m^3 if originally given in ppm.
i      = find(isnan(Xa)==1);
Xa(i)  = pXa(i).*P0(i).*101325.01./(10^6*R*Tka(i));                        %mol/m^3

%Conversion of gas concentration in the water from equivalent atmospheric partial pressure in equilibrium (ppm) to mol/m^3 if originally given in ppm.
i     = find(isnan(Xw)==1);
kH    = estimatekH(P0,S,Tkw,as,bs,kHcp,R,tds,Vb,Vm,NSkH);
kHcp  = 12.1866./(Tka.*kH);                                                %mol/(L*atm)
Xw(i) = pXw(i).*kHcp(i).*P0(i)*10^-3;                                      %mol/m^3

%OBJECTIVES: Estimates friction velocity-----------------------------------
if (sum(strcmp(NSka,{'Joh10' 'M&Y83' 'Jef10'}))==1 || ...
    sum(strcmp(NSkwind,{'Jea87' 'M&Y83' 'MY83w' 'Zea03' 'ZoJ03'}))==1 || ...
    sum(strcmp(NSkbubble,{'W05va'  'W05vw' 'Zhg06' 'ZRb03' 'Z&X10'}))==1)
    
    % preliminary estimate of friction velocity from Eddy-Covariance turbulent fluxes
    if isempty(u_)
        u_=nan(size(U));
    end
    u_ = estimateEC2ustar(u_,uw,vw,zu,NSEC2ustar);

    % preliminary estimate of friction velocity from Drag Coefficient (CD)
    u_ = estimateCD2ustar(U,u_,zu,NSdragcoef);                             %m/s

    %Estimates roughness length
    if isempty(z0)
        z0 = nan(size(u_));
    end   
    z0 = estimatez0(z0,u_,Hs,Lp,fp,A_Ck,B_Ck,Aw,Bw,Cw,g,Karman,z0bounds,NSz0);    %m

    %estimates Bulk Richardson number
    i = find(~isnan(z0));
    if ~isempty(i)
        [Rib,q0,Tp0,Tph,Tv0,Tvh] = estimateRib(P0,Ph,Tw,Tah,hrh,rL0,rLh,qh,U,hTP,g,NSrib,i);
    else
        Rib = [];
    end

    %Estimates Monin-Obhukov Stability theory (MOS)
    if ~isempty(Rib)
        [psi_m,MO_L] = estimateMOS(z0,zu,Rib,Karman,NSmos);
    end
    
    %Estimates friction velocity from Uz, z0 and vertical heat flux or wind log-linear profile
    if isempty(psi_m)
        psi_m = zeros(size(u_));
    end
    psi_m(isnan(psi_m)) = 0;
    
    %Estimates Schimdt number and kinematic viscosity of air
    [Sca,Upsa] = estimateSca(P0,Ta0,Tka,Ma,Mb,Sv,Sd,Va,Vb);
    
    %Iterative estimation of WLLP
    switch NSu_
        case 'WLLP'
            u_ = U.*Karman./(log(zu./z0)+psi_m);

        case 'iWLP'
            try
                for i=1:niWLLP
                    u_ = U.*Karman./(log(zu./z0)+psi_m);
                    z0 = estimatez0(z0,u_,Hs,Lp,fp,A_Ck,B_Ck,Aw,Bw,Cw,g,Karman,z0bounds,NSz0);
                    Re =u_.*z0./Upsa;
                    i  =find(Re<=Re_c(1));
                    j  =find(Re>Re_c(1) & Re<=Re_c(2));
                    
                    switch NSsmoothflow
                        case 'Smi88'
                            z0_   = 0.11*Upsa./u_;
                            z0(i) = z0_(i);
                        case 'Aea11'
                            z0_   = 0.135*Upsa./u_;
                            z0(i) = z0_(i);
                        case 'none.'
                            z0_ = zeros(size(u_));
                    end
                    z0(j) = max(z0(j),z0_(j));
                    
                end

            catch
                'iWLP crashed probably due to lack of Hs and Lp data: try running WLLP'
            end

        case 'VHFx'
            %u_;
    end
    u_(u_<0)=0;

    %estimates U10 from wind log-linear profile (for refference use)
    U10 = U.*(log(10)-log(z0)+psi_m)./(log(zu)-log(z0)+psi_m);

end

%OBJECTIVES: estimates air-side transfer velocity--------------------------

%Estimates air-side transfer velocity
ka = estimateka(psi_m,Sca,U,u_,z0,Karman,Mb,NSka);

clear Aw ans Bw bCOARE Ch cp Cw hTP hr0 hrh i j Karman Ma Mb NSdragcoef NSka NSmos NSrib NSu_ niWLLP P Ph pH2OP Q Re Re_c rL0 rLh Sd Sv Ta0 Tah tds uw vw 
clear -regexp 'CO2'

%% Estimates water-side transfer velocity and gas fluxes
i     = find(zu~=10);
U(i)  = U10(i);
zu(i) = 10;

%Estimates the chemical enhancement factor (alfa) and the Schmidt number of
%the water (Scw)
[Alfa,Scw,Upsw] = estimatealfa(S,Tw,Tkw,NSalfa,NSdw,NSkwind,a,b,c,r,MassFraction,Ms,phiw,Vb);

%Estimates the water-side transfer velocity
[kwind]                    = estimatekwind(U,u_,NSkwind);
[kbubble,W,Rb,Rh,Rhu,beta] = estimatekbubble(R,U,u_,Hs,fp,kHcp,Upsa,Upsw,Scw,g,NSkbubble);
[kcurrent]                 = estimatekcurrent(w,zw,NSkcurrent);
[kw,bt]                    = estimatekw(Tw,chemen,kwind,kbubble,kcurrent,Alfa);


%Estimates the gas flux and total transfer velocity
if strcmp(NSflux,'Mol')==1
    [Fa,Fw,Ka,Kw] = estimateFluxMol(Xa,Xw,kH,ka,kw,NSlayer,NSfluxunits);
elseif strcmp(NSflux,'ppm')==1
    [Fa,Fw,Ka,Kw] = estimateFluxPPM(pXa,pXw,kH,ka,kw,P0,R,Tka,NSlayer,NSfluxunits);
end

clear a as b bs c chemen fp i kHcp MassFraction Ms nc phiw P0 pXa pXw R  r Xa Xw
clear -regexp '^NS|^V'

%% stores the results and clears remaining data

Results.kH           = kH;
try
    Results.z0       = z0;
    Results.q0       = q0;
    Results.qh       = qh;
    Results.Tp0      = Tp0;
    Results.Tph      = Tph;
    Results.Tv0      = Tv0;
    Results.Tvh      = Tvh;
    Results.Rib      = Rib;
    Results.psi_m    = psi_m;
    Results.MO_L     = MO_L;
catch
    'no atmospheric stability implementation'
end
Results.u_           = u_;
Results.U10          = U10;
Results.W            = W;
Results.Upsa         = Upsa;
Results.kwind        = kwind;
Results.Chem_Enh     = bt;
Results.kbubble      = kbubble;
Results.kcurrent     = kcurrent;
Results.Upsw         = Upsw;
Results.Scw          = Scw;
Results.kw           = kw;
Results.Alfa         = Alfa;

try
    Results.beta = beta;
    Results.Rh   = Rh;
    Results.Rhu  = Rhu;
    Results.Rb   = Rb;
catch
    'no kbubble implementation'
end

try
    Results.Kw       = Kw;
    Results.ka       = ka;
    Results.Sca      = Sca;
    Results.Ka       = Ka;
    Results.Flux.air = Fa;
catch
    'single layer transfer velocity: no Ka'
end
Results.Flux.water   = Fw;

clear Alfa beta bt g Hs Lp MO_L psi_m q0 qh Rb Rib Rh Rhu S u_ vhf Wpw w
clear -regexp 'Ck$|^F|^K|^k|^NS|^Sc|^T|^U|^W|^z'

