%TASK: estimate the gas air-water flux and total transfer velocity.

%CALCULUS:-----------------------------------------------------------------

function [Fa,Fw,Ka,Kw] = estimateFluxMol(Ca,Cw,kH,ka,kw,NSlayer,NSfluxunits)

% Flux estimate from Fick's Law--------------------------------------------
switch NSlayer
    
    case 'Single'
        % Air to water flux adapted from Goddard (2006), Zappa et al (2007)
        % Similar to Fw by Johnson 2010
        Fw=kw.*(Cw-Ca./kH);                                    %mol/(m^2*s)   
        Kw=[];
        Ka=[];
        Fa=[];
        
    case 'Double'     
        % Air to water flux adapted from Liss and Slater (1974)
        
        % Total transfer velocity and Flux (air to the water) from the air point of view 
        Ka=(1./ka+kH./kw).^-1;                                        % m/s
        Fa=Ka.*(kH.*Cw-Ca);                                    %mol/(m^2*s)

        % Total transfer velocity and Flux (air to the water) from the water point of view 
        Kw=(1./kw+1./(kH.*ka)).^-1;                                   % m/s
        Fw=Kw.*(Cw-Ca./kH);                                    %mol/(m^2*s)          
end

switch NSfluxunits

    case 2
        Fw=Fw*1000*60*60;                                     %mmol/(m^2*h)
        Fa=Fa*1000*60*60;                                     %mmol/(m^2*h)
    case 3
        Fw=Fw*1000*60*60*24;                                %mmol/(m^2*day)
        Fa=Fa*1000*60*60*24;                                %mmol/(m^2*day)
end
