%TASK: estimate roughness length from wave field height and wave length

%CALCULUS:-----------------------------------------------------------------

function z0 = estimatez0(z0,u_,Hs,Lp,fp,A_Ck,B_Ck,Aw,Bw,Cw,g,karman,z0bounds,NSz0)

switch NSz0
    
    case 'Chk55'

        ir     = find(~isnan(u_));
        u      = u_(ir);                                                   %m/s
        z0(ir) = 0.018*u.^2./g;                                            %m
        
    case 'Ck_cp'
 
        ir     = find(~isnan(Hs) & ~isnan(Lp));
        u      = u_(ir);                                                   %m/s
        Lp_    = Lp(ir);                                                   %m
        fp_    = fp(ir)./(2*pi);                                           %1/s
        c_p    = Lp_.*fp_;                                                 %m/s
        z_Ck   = A_Ck*(c_p./u).^B_Ck;                                      %dimensionless
        z0(ir) = z_Ck.*u.^2./g;                                            %m
        
    case 'TY01a'           % following Taylor and Yelland (2001) \ equation 5

        ir     = find(~isnan(Hs) & ~isnan(Lp));
        Lp_    = Lp(ir);                                                   %m
        Hs_    = Hs(ir);                                                   %m
        z0(ir) = Aw*Hs_.*(Hs_./(Lp_)).^Bw;                                 %m
        
    case 'TY01b'           % following Taylor and Yelland (2001) \ adapted equation 7
        
        ir       = find(~isnan(Hs) & ~isnan(Lp) & ~isnan(u_) & ~isnan(fp));
        u      = u_(ir);                                                   %m/s
        Hs_    = Hs(ir);                                                   %m
        Lp_    = Lp(ir);                                                   %m
        fp_    = fp(ir)./(2*pi);                                           %1/s
        c_p    = Lp_.*fp_;                                                 %m/s
        z0(ir)   = Aw*Hs_.*(Hs_./(Lp_)).^Bw.*(u./c_p).^Cw;                 %m
        
    case 'PanZ0'           % following Pan et al (2008) \ adapted equation 7 for z0

        ir       = find(~isnan(Hs) & ~isnan(Lp) & ~isnan(u_) & ~isnan(fp));
        u      = u_(ir);                                                   %m/s
        Hs_    = Hs(ir);                                                   %m
        Lp_    = Lp(ir);                                                   %m
        fp_    = fp(ir)./(2*pi);                                           %1/s
        c_p    = Lp_.*fp_;                                                 %m/s
        z0(ir)   = Hs.*(u./c_p).^2.82./1.3431;                             %m
        
    case 'PanZC'           % following Pan et al (2008) \ adapted for z_Ch
        
        ir       = find(~isnan(Lp) & ~isnan(u_) & ~isnan(fp));
        u      = u_(ir);                                                   %m/s
        Lp_    = Lp(ir);                                                   %m
        fp_    = fp(ir)./(2*pi);                                           %1/s
        c_p    = Lp_.*fp_;                                                 %m/s
        z_Ck   = (u./c_p).^1.4./1.0618;                                    %m
        z0(ir) = z_Ck.*u.^2./g;                                            %m
        
   case 'Hwg05'           % following Hwang (2005) \ adapted for any water depth
        
        ir       = find(~isnan(Lp) & ~isnan(u_) & ~isnan(fp));
        u      = u_(ir);                                                   %m/s
        Lp_    = Lp(ir);                                                   %m
        fp_    = fp(ir)./(2*pi);                                           %1/s
        c_p    = Lp_.*fp_;                                                 %m/s
        z0(ir) = Lp_./2.*exp(-karman*(0.0122*(u./c_p).^0.704).^-0.5);     %m 
        
end
        
i     = find(~isnan(z0));
z0(i) = max(z0bounds(1),min(z0bounds(2),z0(i)));


