                            %% CONSTANTS
                   
% Henry's law constant in pure water at 25Celsius. K=c/p form.            mol/(L.atm)
kHcp = 0.034;      %(for CO2)
%kHcp = 0.0014;    %(for CH4)
%kHcp = 0.025;     %(for N2O)

% Relative molecular mass of air
Ma = 28.97;

% Relative molecular mass
Mc = 44.01;           %(for CO2)
%Mc = 16.043;           %(for CH4)
%Mc = 44.013;           %(for N2O)

% Relative molecular mass of H20
Mv=18.01;

% Mass fraction (MF) of solutes in standard sea water as a proportion of
% total salinity.
MassFraction= ...
{'Compound' 'MF'  'v1'     'v2'      'v3'     'v4'       'v5'      'v6'   ;
  'NaCl'    0.798  16.22    1.3229    1.4849   0.0074691  30.78    2.0583 ;
  'KCl'     0.022   6.4883  1.3175   -0.7785   0.09272    -1.3     2.0811 ;
  'CaCl2'   0.033  32.028   0.787792 -1.1495   0.0026995  780860   5.8442 ;
  'MgCl2'   0.047  24.032   2.2694    3.7108   0.021853   -1.1236  0.14474;
  'MgSO4'   0.100  72.269   2.2238    6.6037   0.0079004  3340.1   6.1304};

% Association factor of H20
phiw = 2.6;

% Universal ideal gas law constant
R = 8.314472;

% gravitational acceleration                                          m/s^2
g = 9.80665;

% Rhow0 (pure water) parameters (in Johnson 2010)
r = [999.842594 0.06793952 -0.00909529 0.0001001685 -0.000001120083 0.000000006536332];

% Rhow (marine water) parameters (in Johnson 2010)
a = [0.824493 -0.0040899 0.000076438 -0.00000082467 0.0000000053875];
b = [-0.00572466 0.00010277 -0.0000016546];
c  = 0.00048314;

% Rho air parameters (in Johnson 2010)
Sd = [1.293393662 -5.538444326*10^-3 3.860201577*10^-5 -5.2536065*10^-7];

% Eta air parameters (in Johnson 2010)
Sv = [1.715747771*10^-5 4.722402075*10^-8 -3.663027156*10^-10 ...
      1.873236686*10^-12 -8.050218737*10^-14];

%'a' and 'b' constants for temperature and salinity dependence of Bunsen
% solubility coeficient taken from Sarimiento (2013)
as = [-160.7373   215.4152  89.892    -1.477592]; %for CO2
bs = [ 0.029941  -0.027455  0.0053407          ]; %for CO2
%as = [-68.8862    101.4956  28.7314   0        ]; %for CH4
%bs = [-0.076146   0.04397  -0.0068672          ]; %for CH4
%as = [-165.8806   222.8743  92.0792   -1.48425 ]; %for N2O
%bs = [-0.056235   0.031619 -0.0048472          ]; %for N2O


% Temperature dependence of the solution: -delta_soln*H/R (in Johnson 2010)
tds = 2400;         %(for CO2)
%tds = 1700;         %(for CH4)
%tds = 2600;         %(for N2O)

% Liquid molar volume of air                                               cm^3/mol
Va = 20.1;   

% Liquid molar volume at its boiling point using the additive Schroeder method:
% increments of C=7,O=7, N=7, H=7, double bound=7 and triple bound=14 (see Johnson 2010)
% for CO2: molecular structure of CO2 is O=C=O
Vb = 1*7+2*7+2*7;                                                 %cm^3/mol
% for CH4: molecular structure of CH4 is 1C to 4H with 4 single bounds
%Vb = 1*7+4*7;
% for N2O: molecular structure of N2O is N=N=O
%Vb = 1*7+2*7+2*7;                                                 %cm^3/mol

%Real gas molar volume
Vm = 22.3;  %for CO2 and CH4                                         %L/mol
%Vm = 22.2432;  %for N2O                                              %L/mol

        %% FLUX NUMERICAL OPTIONS ( required for later k estimations)

% Flux quantity 'moles' for mol.m^-2.s^-1 or 'grams' g.m^-2.s^-1
fquant = 'moles';

% Henry constant correction for temperature and salinity
% 'Sar13': Sarmiento (2013) adapted from Weiss(1974), Weiss and Price(1980)
% 'Joh10': Johnson (2010)
NSkH = 'Sar13';
        
% Water transfer velocity (kwind)
NSkwind = 'Zea03';

% Chemical enhancement of water transfer velocity (alfa)
% 'Bea04': Borges et al (2004)
% 'Cea96': Carini et al (1996)
% 'Joh10': Johnson (2010)
% 'R&C01': Raymond Cole (2001)
NSalfa = 'Joh10';

% Water difusivity scheme (Dw)
% 'H&L74': Hayduk & Laudie (1974)
% 'H&M82': Hayduk & Minhas (1982)
% 'W&C55': Wilkie & Chang  (1955)
NSdw = 'W&C55';

              %% Eddy-Covariance Estimated Gas Fluxes
              
%load Eddy-Covariance data
dir ='C:\Users\Vasco Vieira\Dropbox\InterfaceGasFlux\Dados\Baltico\';
file='Upsalla2015';
load([dir file]);
stat = data201509.ten;

rssi=stat.rssi_rssi; cccc=stat.c_c_c_c; tttt=stat.ts_ts_ts_ts; ww=stat.w_w; WD=stat.WD;
WDa=[WD(2:end); NaN]; WDb=[NaN; WD(1:end-1)];

i   = find(rssi>0.001 | cccc>100 | ww<0.0000011 | tttt>0.1 | WD<80 | WDb<80 | WDa<80 | WD>160 | WDb>160 | WDa>160 );

C=stat.C;          C(i)=nan;   C=mean([C(1:end-2)     C(2:end-1)   C(3:end)],2);            
CD=stat.CD;       CD(i)=nan;  CD=mean([CD(1:end-2)   CD(2:end-1)  CD(3:end)],2);
P=stat.PA;         P(i)=nan;   P=mean([P(1:end-2)     P(2:end-1)   P(3:end)],2);
Q=stat.Q;          Q(i)=nan;   Q=mean([Q(1:end-2)     Q(2:end-1)   Q(3:end)],2);
QD=stat.QD;       QD(i)=nan;  QD=mean([QD(1:end-2)   QD(2:end-1)  QD(3:end)],2);        
T=stat.TS;         T(i)=nan;   T=mean([T(1:end-2)     T(2:end-1)   T(3:end)],2);            
WS=stat.WS;       WS(i)=nan;  WS=mean([WS(1:end-2)   WS(2:end-1)  WS(3:end)],2);        
wcd=stat.w_cd;   wcd(i)=nan; wcd=mean([wcd(1:end-2) wcd(2:end-1) wcd(3:end)],2);    
wqd=stat.w_qd;   wqd(i)=nan; wqd=mean([wqd(1:end-2) wqd(2:end-1) wqd(3:end)],2);    
wc=stat.w_c;      wc(i)=nan;  wc=mean([wc(1:end-2)   wc(2:end-1)  wc(3:end)],2);
wq=stat.w_q;      wq(i)=nan;  wq=mean([wq(1:end-2)   wq(2:end-1)  wq(3:end)],2);
wt=stat.w_ts;     wt(i)=nan;  wt=mean([wt(1:end-2)   wt(2:end-1)  wt(3:end)],2);
date=stat.date;
dnum = datenum(date)-1/24;  %converting date from SNT to UTC 

%load measured environmental variables
dir ='C:\Users\Vasco Vieira\Dropbox\InterfaceGasFlux\Dados\Baltico\';
file='SAMI_CO2_2015';
load([dir file]);
dnumSAMI = datenum(cell2mat(DatetimeUTC))-1/24;  %converting date from SNT to UTC 
file='Seabird_2015';
load([dir file]);
dnumSeabird = datenum(cell2mat(DateSeabird))-1/24;
file='DWR_gotl_09_2015';
load([dir file]);
dnumDWR = datenum([YY MM DD UTC zeros(size(YY))]);

s1 = size(C);
select1=nan(s1);
select2=nan(s1);
select3=nan(s1);
for j=1:s1(1)
    try
        i          = find(dnumSAMI==dnum(j));
        select1(j) = i;
        i          = find(dnumSeabird==dnum(j));
        select2(j) = i;
        i          = find(dnumDWR==dnum(j));
        select3(j) = i;
    catch
        continue
    end
end

i           = find(~isnan(select1) & ~isnan(select2) & ~isnan(select3) & select3>0);
date        = date(i,:);
dnum        = dnum(i);
C           = C(i);
CD          = CD(i);
P           = P(i);
Q           = Q(i);
QD          = QD(i);
T           = T(i);
WD          = WD(i);
WS          = WS(i);
wc          = wc(i);
wcd         = wcd(i);
wq          = wq(i);
wqd         = wqd(i);
wt          = wt(i);
select1     = select1(i);
DatetimeUTC = DatetimeUTC(select1);
CO2ppm      = CO2ppm(select1);
TwSAMI      = TwSAMI(select1);
select2     = select2(i);
DateSeabird = DateSeabird(select2);
Depth       = Depth(select2);
S           = Salinity(select2);
TwSeabird   = TwSeabird(select2);
select3     = select3(i);
fp          = fp(select3);
Hs          = Hs(select3);
Lp          = Lp(select3);
TwDWR       = SST(select3);

K  = T+273.15;
P  = P*1000;
mu = Ma/Mv;

%uncomment the lines relative to the variables you are using & comment the lines relative to the variables you are not using
rho  = P*Ma./(R*K);
rhov = QD*Mv./1000;                                                       %a
%rhov = Q.*rho./1000;                                                       %b
rhoc = CD*Mc./1000;                                                       %a
%rhoc = C.*P*Mc./(10^6*R*K);                                                %b
rhoa = rho-rhov-rhoc;

switch fquant
    case 'moles'
        %F1   = wc.*P./(10^6*R*K);                                          %b
        F1   = wcd./1000;                                                  %a
        %F2   = mu.*rhoc./rhoa.*rhoa.*wq./(1000*Mv);                         %b
        F2   = mu.*rhoc./rhoa.*wqd./1000;                                  %a
        F3   = (1+mu*rhov./rhoa).*rhoc.*wt./(K*Mc);

    case 'grams'
        %F1   = wc.*P*Mc./(10^6*R*K);                                      %b
        F1   = wcd*Mc./1000;                                               %a
        %F2   = mu.*rhoc./rhoa.*rhoa.*wq./1000;                             %b
        F2   = mu.*rhoc./rhoa.*wqd./1000*Mv;                               %a
        F3   = (1+mu*rhov./rhoa).*rhoc./K.*wt;                             
end

F = F1+F2+F3;   % When using w'c', F=F1

                      %% DATA (for k estimates)

% Gas concentration in the air in                                   mol/m^3
Xa = nan(size(C));%Ca_mmol./1000;

% Gas concentration in the air in                                       ppm
pca = C;

% Air temperature at height 0                                          (C)
Ta0 = TwDWR;

% Air temperature at height h                                          (C)
Tah = T;

% Air pressure at height 0                                            (atm)
%P0  = pa/1000;

% Air pressure at height h                                            (atm)
%Ph  = pa/1000;

% relative humidity at height h                                    (scalar)
hrh = Q;

% Wind velocity h meters above the surface                              m/s
U = WS;

% height of wind measurement                                            (m)
zu = 12*ones(size(C));

% Gas concentration in the water                                    mol/m^3
Xw = nan(size(C));

% Gas concentration in the water in                                     ppm
pcw = CO2ppm;

% Salinity                                                              ppt
S = S;

% Water skin temperature                                                 C
Tw = TwDWR;

% Water current velocity                                              (m/s)
w = 0.001*ones(size(C));

% Water depth                                                           (m)
zw = Depth;

                             %% k CALCULUS

%observed transfer velocity
Tka   = 273.15+Ta0;
Tkw   = 273.15+Tw;
kH    = estimatekH(P/101325.01,S,Tkw,as,bs,kHcp,R,tds,Vb,Vm,NSkH);
pH2OP = exp(24.4543-67.4509*(100./Tkw)-4.8489*log(Tkw./100)-0.000544*S);
pca   = pca.*(1-pH2OP);
dpc   = pcw.*kH-pca;

switch fquant
    case 'moles'
    k     = 10^6*F.*R.*K./(dpc.*P);                                        %m/s
    case 'grams'
    k     = 10^6*F.*R.*K./(dpc.*P*Mc);                                     %m/s   
end

ibounds = find(abs(dpc)>50 & isnan(F)==0 & Lp.*fp./WS<5);                             

                %% plot for E-C script 'get_EC_gasflux.m'
                
dmin = datenum([2015 9 0 0 0 0]);

figure(1)
subplot (2,3,1)
plot(dnum-dmin,F1,'k.'); hold on
plot(dnum-dmin,F2,'b.'); hold on
plot(dnum-dmin,F3,'r.'); hold on
ylabel('WPL')
legend('F1','F2','F3')

subplot (2,3,2)
plot(dnum-dmin,F,'r.'); hold on
plot(dnum(ibounds)-dmin,F(ibounds),'k.'); hold on
ylabel('F(CO_2)')

subplot(2,3,3)
plot(dnum-dmin,Q,'r.'); hold on
plot(dnum(ibounds)-dmin,Q(ibounds),'k.'); hold on
ylabel('q (g/Kg)')

subplot(2,3,4)
plot(dnum-dmin,WS,'r.'); hold on
plot(dnum(ibounds)-dmin,WS(ibounds),'k.'); hold on
xlabel('date num')
ylabel('wind speed')

subplot(2,3,5)
plot(dnum-dmin,WD,'r.'); hold on
plot(dnum(ibounds)-dmin,WD(ibounds),'k.'); hold on
xlabel('date num')
ylabel('wind direction')

subplot(2,3,6)
plot(dnum-dmin,dpc,'r.'); hold on
plot(dnum(ibounds)-dmin,dpc(ibounds),'k.'); hold on
xlabel('date num')
ylabel('\Deltappm')

                 %% figure with transfer velocities
                 
Alfa  = estimatealfa(S,Tw,Tkw,NSalfa,NSdw,NSkwind,a,b,c,r,MassFraction,Mv,phiw,Vb);
k600  = k*100*60*60./Alfa;
                
figure(2)
subplot(2,2,1)
plot(U(ibounds),k(ibounds)*100*60*60,'k.'); hold on
xlabel('U_z')
ylabel('k (cm/h)')

subplot(2,2,2)
plot(U(ibounds),k600(ibounds),'k.'); hold on
xlabel('U_z')
ylabel('k_6_0_0 (cm/h)')

subplot(2,2,3)
plot(dnum(ibounds)-dmin,k(ibounds)*100*60*60,'k.'); hold on
xlabel('date num')
ylabel('k (cm/h)')

subplot(2,2,4)
plot(dnum(ibounds)-dmin,k600(ibounds),'k.'); hold on
xlabel('date num')
ylabel('k_6_0_0 (cm/h)')

clear Alfa a as b bs DD file fp g i j Hs Karman kHpc Lp mu p QD q qh YY;
clear ('-regexp','^C|^c|^d|^F\d|^h|^M|^N|^R|^r|^S|^s|^P|^p|^T|^t|^V|^W|^w|^X|^z') 
